#pragma once

#include <a8/asiotcpclient.h>

#ifdef USE_BOOST

#include <asio.hpp>

using asio::ip::tcp;

namespace a8
{

    class WebSocketClient
    {
    public:
        WebSocketClient(std::shared_ptr<asio::io_context> io_context, const std::string& remote_ip, int remote_port);
        virtual ~WebSocketClient();

        std::function<void (a8::WebSocketClient*, int)> on_error;
        std::function<void (a8::WebSocketClient*)> on_connect;
        std::function<void (a8::WebSocketClient*, int)> on_disconnect;
        std::function<void (char*, int&, unsigned int)> on_decode_userpacket;

        void Open();
        void Close();
        bool IsActive();
        bool Connected();
        void SendBuff(const char* buff, unsigned int bufflen);

    private:
        void DecodePacket(char* buf, int& offset, unsigned int buflen);

        void ProcessHandShake(char* buf, int& offset, unsigned int buflen);
        void ProcessWsHandShake(char* buf, int& offset, unsigned int buflen);
        void ProcessUserPacket();
        void DecodeFrame(char* buf, int& offset, unsigned int buflen);

    private:
        std::shared_ptr<AsioTcpClient> tcp_client_;
        char *decoded_buff_ = nullptr;
        int decoded_bufflen_ = 0;
        bool handshook_ = false;
        char *recv_buff_ = nullptr;
        int recv_bufflen_ = 0;
        int max_packet_len_ = 0;
    };

}

#endif
